@@ -1,6 +1,6 @@
 
 """
-    Copyright (C) 2017 SunSpec Alliance
+    Copyright (C) 2018 SunSpec Alliance
 
     Permission is hereby granted, free of charge, to any person obtaining a
     copy of this software and associated documentation files (the "Software"),
@@ -23,8 +23,10 @@
 
 import os
 import time
+import struct
+import sys
 import sunspec.core.modbus.client as modbus
-import sunspec.core.device as device 
+import sunspec.core.device as device
 import sunspec.core.util as util
 import sunspec.core.suns as suns
 from sunspec.core.util import SunSpecError
@@ -41,9 +43,85 @@ class SunSpecClientError(SunSpecError):
 
 class ClientDevice(device.Device):
 
+    """ClientDevice
+
+    A derived class based on :const:`sunspec.core.device.Device`. It adds Modbus
+    device access capability to the device base class.
+
+    Parameters:
+
+        device_type :
+             Device type. Possible values: :const:`RTU`, :const:`TCP`,
+             :const:`MAPPED`.
+
+        slave_id :
+            Modbus slave id.
+
+        name :
+            For :const:`RTU` devices, the name of the serial port such as 'com4'
+            or '/dev/tty2'. For :const:`MAPPED` devices, the name of the modbus
+            map file.
+
+        pathlist :
+            Pathlist object containing alternate paths to support files.
+
+        baudrate :
+            For :const:`RTU` devices, baud rate such as 9600 or 19200. Defaulted
+            by modbus module to 9600.
+
+        parity :
+            For :const:`RTU` devices, parity. Possible values:
+            :const:`PARITY_NONE`, :const:`PARITY_EVEN`
+            Defaulted by modbus module to :const:`PARITY_NONE`.
+
+        ipaddr :
+            For :const:`TCP` devices, device IP address.
+
+        ipport :
+            For :const:`TCP` devices, device IP port. Defaulted by modbus module
+            to 502.
+
+        timeout :
+            Modbus request timeout in seconds. Fractional seconds are permitted
+            such as .5.
+
+        trace :
+            Enable low level trace.
+
+    Raises:
+
+        SunSpecClientError: Raised for any sunspec module error.
+
+    Attributes:
+
+        type
+            Device type. Possible values: :const:`RTU`, :const:`TCP`,
+            :const:`MAPPED`.
+
+        name
+            For :const:`RTU` devices, the name of the serial port such as 'com4'
+            or '/dev/tty2'. For :const:`MAPPED` devices, the name of the modbus
+            map file.
+
+        pathlist
+            Pathlist object containing alternate paths to support files.
+
+        slave_id
+            Modbus slave id.
+
+        modbus_device
+            Modbus device object. Object type is based on the device type.
+
+        retry_count
+            Request retry count. Currently not used.
+
+        base_addr_list
+            List of Modbus base addresses to try when scanning a device for the
+            first time.
+    """
+
     def __init__(self, device_type, slave_id=None, name=None, pathlist = None, baudrate=None, parity=None, ipaddr=None, ipport=None,
                  timeout=None, trace=False):
-
         device.Device.__init__(self, addr=None)
 
         self.type = device_type
@@ -66,7 +144,7 @@ class ClientDevice(device.Device):
                     if self.modbus_device is not None:
                         self.modbus_device.close()
                     raise SunSpecClientError('Map file required for mapped device')
-        except modbus.ModbusClientError, e:
+        except modbus.ModbusClientError as e:
             if self.modbus_device is not None:
                 self.modbus_device.close()
             raise SunSpecClientError('Modbus error: %s' % str(e))
@@ -77,30 +155,61 @@ class ClientDevice(device.Device):
             self.modbus_device.close()
 
     def read(self, addr, count):
+        """Read Modbus device registers.
+
+        Parameters:
+
+            addr :
+                Starting Modbus address.
+
+            count :
+                Register count.
+
+        Returns:
+            Byte string containing register contents.
+        """
+
         try:
             if self.modbus_device is not None:
                 return self.modbus_device.read(addr, count)
             else:
                 raise SunSpecClientError('No modbus device set for SunSpec device')
-        except modbus.ModbusClientError, e:
+        except modbus.ModbusClientError as e:
             raise SunSpecClientError('Modbus read error: %s' % str(e))
 
     def write(self, addr, data):
+        """Write Modbus device registers.
+
+        Parameters:
+
+            addr :
+                Starting Modbus address.
+
+            count :
+                Byte string containing register contents.
+        """
 
         try:
             if self.modbus_device is not None:
                 return self.modbus_device.write(addr, data)
             else:
                 raise SunSpecClientError('No modbus device set for SunSpec device')
-        except modbus.ModbusClientError, e:
+        except modbus.ModbusClientError as e:
             raise SunSpecClientError('Modbus write error: %s' % str(e))
 
     def read_points(self):
+        """Read the points for all models in the device from the physical
+        device.
+        """
 
         for model in self.models_list:
             model.read_points()
 
     def scan(self, progress=None, delay=None):
+        """Scan all the models of the physical device and create the
+        corresponding model objects within the device object based on the
+        SunSpec model definitions.
+        """
 
         error = ''
 
@@ -114,17 +223,16 @@ class ClientDevice(device.Device):
 
         if self.base_addr is None:
             for addr in self.base_addr_list:
-                # print 'trying base address %s' % (addr)
+                # print('trying base address %s' % (addr))
                 try:
                     data = self.read(addr, 3)
-
-                    if data[:4] == 'SunS':
+                    if data[:4] == b'SunS':
                         self.base_addr = addr
-                        # print 'device base address = %d' % self.base_addr
+                        # print('device base address = %d' % self.base_addr)
                         break
                     else:
                         error = 'Device responded - not SunSpec register map'
-                except SunSpecClientError, e:
+                except SunSpecClientError as e:
                     if not error:
                         error = str(e)
 
@@ -132,7 +240,7 @@ class ClientDevice(device.Device):
                     time.sleep(delay)
 
         if self.base_addr is not None:
-            # print 'base address = %s' % (self.base_addr)
+            # print('base address = %s' % (self.base_addr))
             model_id = util.data_to_u16(data[4:6])
             addr = self.base_addr + 2
 
@@ -146,14 +254,14 @@ class ClientDevice(device.Device):
                         if not cont:
                             raise SunSpecClientError('Device scan terminated')
                     model_len = util.data_to_u16(data)
-                    # print 'model_id = %s  model_len = %s' % (model_id, model_len)
+                    # print('model_id = %s  model_len = %s' % (model_id, model_len))
 
                     # move address past model id and length
                     model = ClientModel(self, model_id, addr + 2, model_len)
-                    # print 'loading model %s at %d' % (model_id, addr + 2)
+                    # print('loading model %s at %d' % (model_id, addr + 2))
                     try:
                         model.load()
-                    except Exception, e:
+                    except Exception as e:
                         model.load_error = str(e)
                     self.add_model(model)
 
@@ -178,16 +286,45 @@ class ClientDevice(device.Device):
             self.modbus_device.disconnect()
 
 class ClientModel(device.Model):
+    """A derived class based on :const:`sunspec.core.device.Model`. It adds
+    Modbus device access capability to the model base class.
+
+    Parameters:
+
+        dev :
+            Device object associated with the model.
+
+        mid :
+            Model id.
+
+        addr :
+            Starting Modbus address of the model.
+
+        mlen :
+            Model length in Modbus registers.
+
+        index :
+            Model index.
+
+    Raises:
+
+        SunSpecClientError: Raised for any sunspec module error.
+    """
 
     def __init__(self, dev=None, mid=None, addr=0, mlen=None, index=1):
 
         device.Model.__init__(self, device=dev, mid=mid, addr=addr, mlen=mlen, index=index)
 
     def load(self):
+        """Create the block and point objects within the model object based on
+        the corresponding SunSpec model definition.
+        """
 
         device.Model.load(self, block_class=ClientBlock, point_class=ClientPoint)
 
     def read_points(self):
+        """Read all points in the model from the physical device.
+        """
 
         if self.model_type is not None:
             # read current model
@@ -196,7 +333,7 @@ class ClientModel(device.Model):
                 if end_index == 1:
                     data = self.device.read(self.addr, self.len)
                 else:
-                    data = ''
+                    data = b''
                     index = 0
                     while index < end_index:
                         addr = self.read_blocks[index]
@@ -207,7 +344,7 @@ class ClientModel(device.Model):
                             read_len = self.addr + self.len - addr
                         data += self.device.read(addr, read_len)
                 if data:
-                    # print 'data len = ', len(data)
+                    # print('data len = ', len(data))
                     data_len = len(data)/2
                     if data_len != self.len:
                         raise SunSpecClientError('Error reading model %s' % self.model_type)
@@ -215,11 +352,11 @@ class ClientModel(device.Model):
                     #  for each repeating block
                     for block in self.blocks:
                         # scale factor points
-                        for pname, point in block.points_sf.iteritems():
+                        for pname, point in block.points_sf.items():
                             offset = int(point.addr) - int(self.addr)
                             if point.point_type.data_to is not None:
                                 byte_offset = offset * 2
-                                # print pname, point, offset, byte_offset, (byte_offset + (int(point.point_type.len) * 2)), point.point_type.len
+                                # print(pname, point, offset, byte_offset, (byte_offset + (int(point.point_type.len) * 2)), point.point_type.len)
                                 point.value_base = point.point_type.data_to(data[byte_offset:byte_offset + (int(point.point_type.len) * 2)])
                                 if not point.point_type.is_impl(point.value_base):
                                     point.value_base = None
@@ -227,12 +364,15 @@ class ClientModel(device.Model):
                                 raise SunSpecClientError('No data_to function set for %s : %s' % (pname, point.point_type))
 
                         # non-scale factor points
-                        for pname, point in block.points.iteritems():
+                        for pname, point in block.points.items():
                             offset = int(point.addr) - int(self.addr)
                             if point.point_type.data_to is not None:
                                 byte_offset = offset * 2
-                                # print pname, point, offset, byte_offset, (byte_offset + (int(point.point_type.len) * 2)), point.point_type.len
+                                # print(pname, point, offset, byte_offset, (byte_offset + (int(point.point_type.len) * 2)), point.point_type.len)
                                 point.value_base = point.point_type.data_to(data[byte_offset:byte_offset + (int(point.point_type.len) * 2)])
+                                if (type(point.value_base) == bytes and
+                                        sys.version_info > (3,)):
+                                    point.value_base = str(point.value_base, 'latin-1')
                                 if point.point_type.is_impl(point.value_base):
                                     if point.sf_point is not None:
                                         point.value_sf = point.sf_point.value_base
@@ -242,18 +382,21 @@ class ClientModel(device.Model):
                             else:
                                 raise SunSpecClientError('No data_to function set for %s : %s' % (pname, point.point_type))
 
-            except SunSpecError, e:
+            except SunSpecError as e:
                 raise SunSpecClientError(e)
-            except modbus.ModbusClientError, e:
+            except modbus.ModbusClientError as e:
                 raise SunSpecClientError('Modbus error: %s' % str(e))
             except:
                 raise
 
     def write_points(self):
+        """Write all points that have been modified since the last write
+        operation to the physical device.
+        """
 
         addr = None
         next_addr = None
-        data = ''
+        data = b''
 
         for block in self.blocks:
             for point in block.points_list:
@@ -263,12 +406,12 @@ class ClientModel(device.Model):
                     point_data = point.point_type.to_data(point.value_base, (point_len * 2))
                     if addr is None:
                         addr = point_addr
-                        data = ''
+                        data = b''
                     else:
                         if point_addr != next_addr:
                             block.model.device.write(addr, data)
                             addr = point_addr
-                            data = ''
+                            data = b''
                     next_addr = point_addr + point_len
                     data += point_data
                     point.dirty = False
@@ -277,18 +420,65 @@ class ClientModel(device.Model):
                 addr = None
 
 class ClientBlock(device.Block):
+    """A derived class based on :const:`sunspec.core.device.Block`. It adds
+    Modbus device access capability to the block base class.
+
+    Parameters:
+
+        model :
+            Model object associated with the block.
+
+        addr :
+            Starting Modbus address of the block.
+
+        blen :
+            Block length in Modbus registers.
+
+        block_type :
+            The block type object associated with block in the model
+            definition.
+
+        index :
+            Block index.
+    """
 
     def __init__(self, model, addr, blen, block_type, index=1):
 
         device.Block.__init__(self, model, addr, blen, block_type, index)
 
 class ClientPoint(device.Point):
+    """A derived class based on :const:`sunspec.core.device.Point`. It adds
+    Modbus device access capability to the point base class.
+
+    Parameters:
+
+        block :
+            Block object associated with the point.
+
+        point_type :
+            The point type object associated with point in the model definition.
+
+        addr :
+            Starting Modbus address of the point.
+
+        sf_point :
+            Point object associated with the point scale factor if present.
+
+        value :
+            Point value.
+
+    Raises:
+
+        SunSpecClientError: Raised for any sunspec module error.
+    """
 
     def __init__(self, block=None, point_type=None, addr=None, sf_point=None, value=None):
 
         device.Point.__init__(self, block, point_type, addr, sf_point, value)
 
     def write(self):
+        """Write the point to the physical device.
+        """
 
         data = self.point_type.to_data(self.value_base, (int(self.point_type.len) * 2))
         self.block.model.device.write(int(self.addr), data)
@@ -296,6 +486,43 @@ class ClientPoint(device.Point):
 
 class SunSpecClientModelBase(object):
 
+    """This class forms the base class of the dynamically generated model
+    classes during SunSpecClientDevice initialization. In addition to the
+    attributes listed below, the model (fixed block) points are placed as
+    attributes on the model.
+
+    Parameters:
+
+        model :
+            The :const:`sunspec.core.device.Model` associated with the model.
+
+        name :
+            Model name as specified in the model definition.
+
+    Raises:
+
+        SunSpecClientError : Raised for any sunspec module error.
+
+    Attributes:
+
+        model
+            The :const:`sunspec.core.device.Model` object associated with the
+            model.
+
+        name
+            Model name as specified in the model definition.
+
+        repeating
+            Repeating block if the model contains one.
+
+        repeating_name
+            Repeating block name.
+
+        points
+            Names of the point attributes added to the model object.
+    """
+
+
     def __init__(self, model, name):
         self.model = model
         self.name = name
@@ -326,16 +553,21 @@ class SunSpecClientModelBase(object):
 
     def __getitem__(self, name):
         return self._get_property(name)
-        # return self.__dict__.get(key, None)
+        # return self.__dict__.get(name, None)
 
     def __setitem__(self, name, item):
         return self._set_property(name, item)
-        # self.__dict__.set(key, item)
+        # self.__dict__.set(name, item)
 
     def read(self):
+        """Read all points in the model from the physical device."""
+
         self.model.read_points()
 
     def write(self):
+        """Write all points that have been modified since the last write
+        operation to the physical device."""
+
         self.model.write_points()
 
     def __str__(self):
@@ -352,6 +584,35 @@ class SunSpecClientModelBase(object):
 
 class SunSpecClientBlockBase(object):
 
+    """SunSpecClientBlockBase
+
+    This class forms the base class of the dynamically generated repeating
+    block classes during SunSpecClientDevice initialization. In addition to
+    the attributes listed below, the repeating block points are placed as
+    attributes on the repeating block.
+
+    Parameters:
+
+        block :
+             The :const:`sunspec.core.device.Block` object associated with the
+             block.
+
+        name :
+             Repeating block name as specified in the model definition.
+
+    Attributes:
+
+        block
+            The :const:`sunspec.core.device.Block` object associated with the
+            block.
+
+        name
+            Block name as specified in the model definition.
+
+        points
+            Names of the point attributes added to the block object.
+    """
+
     def __init__(self, block, name):
         self.block = block
         self.name = name
@@ -366,13 +627,13 @@ class SunSpecClientBlockBase(object):
         if point:
             point.value = value
 
-    def __getitem__(self, key):
+    def __getitem__(self, name):
         return self._get_property(name)
-        # return self.__dict__.get(key, None)
+        # return self.__dict__.get(name, None)
 
-    def __setitem__(self, key, item):
+    def __setitem__(self, name, item):
         return self._set_property(name, item)
-        # self.__dict__.set(key, item)
+        # self.__dict__.set(name, item)
 
     def __str__(self):
         s = '\n%s[%d]:\n' % (self.name, self.block.index)
@@ -380,7 +641,7 @@ class SunSpecClientBlockBase(object):
             value = getattr(self, name)
             if value is not None:
                 s += '%s:  %s\n' % (name, str(value))
-    
+
         return s
 
 def model_class_get(model_id):
@@ -406,7 +667,7 @@ def model_class_get(model_id):
     model_type = None
     try:
         model_type = device.model_type_get(model_id)
-    except Exception, e:
+    except Exception as e:
         setattr(class_, 'load_error', str(e))
     if model_type is not None:
         for point_type in model_type.fixed_block.points_list:
@@ -431,6 +692,77 @@ def model_class_get(model_id):
 
 class SunSpecClientDevice(object):
 
+    """This class wraps the sunspec.core.ClientDevice class to provide an
+    alternate syntax for scripting. By placing the model (fixed block) points,
+    and repeating block points directly on the model and repeating block objects
+    as attributes, the syntax for accessing them is simplified.
+
+    The model and block classes within the device are dynamically generated
+    based on the model type with the appropriate attributes being added during
+    creation.
+
+    Parameters:
+
+        device_type :
+            Device type. Possible values: :const:`RTU`, :const:`TCP`,
+            :const:`MAPPED`.
+
+        slave_id :
+            Modbus slave id
+
+        name :
+            For :const:`RTU` devices, the name of the serial port such as 'com4'
+            or '/dev/ttyUSB0'. For :const:`MAPPED` devices, the name of the
+            modbus map file.
+
+        pathlist :
+            Pathlist object containing alternate paths to support files.
+
+        baudrate :
+            For :const:`RTU` devices, baud rate such as 9600 or 19200. Defaulted
+            by modbus module to 9600.
+
+        parity :
+            For :const:`RTU` devices, parity. Possible values:
+            :const:`sunspec.core.client.PARITY_NONE`,
+            :const:`sunspec.core.client.PARITY_EVEN` Defaulted by modbus module
+            to :const:`PARITY_NONE`.
+
+        ipaddr :
+            For :const:`TCP` devices, device IP address.
+
+        ipport :
+            For :const:`TCP` devices, device IP port. Defaulted by modbus module
+            to 502.
+
+        timeout :
+            Modbus request timeout in seconds. Fractional seconds are permitted
+            such as .5.
+
+        trace :
+            Enable low level trace.
+
+    Raises:
+
+        SunSpecClientError: Raised for any sunspec module error.
+
+    Attributes:
+
+        device
+            The :const:`sunspec.core.client.ClientDevice` associated with this
+            object.
+
+        models
+            List of models present in the device in the order in which they
+            appear in the device. If there is a single instance of the model in
+            the device, the list element is a model object.
+
+            If there are multiple instances of the same model in the list, the
+            list element for that model is a list of the models of that type in
+            the order in which they appear in the device with the first element
+            having an index of 1.
+    """
+
     def __init__(self, device_type, slave_id=None, name=None, pathlist = None, baudrate=None, parity=None, ipaddr=None, ipport=None,
                  timeout=None, trace=False, scan_progress=None, scan_delay=None):
 
@@ -465,15 +797,23 @@ class SunSpecClientDevice(object):
                 else:
                     setattr(self, name, model_class)
                     self.models.append(name)
-        except Exception, e:
+        except Exception as e:
             if self.device is not None:
                 self.device.close()
             raise
 
     def close(self):
+        """Release resources associated with the device. Should be called when
+        the device object is no longer in use.
+        """
+
         self.device.close()
 
     def read(self):
+        """Read the points for all models in the device from the physical
+        device.
+        """
+
         self.device.read_points()
 
     def __getitem__(self, key):
@@ -481,7 +821,7 @@ class SunSpecClientDevice(object):
 
     def __setitem__(self, key, item):
         self.__dict__.set(key, item)
-        
+
     def __str__(self):
 
         s = ''
